use std::ops::MulAssign;

use ndarray::{Array1, ArrayView1};

pub enum Type {
    MSE,
    MAE
}

pub fn parse_type(t: Type) -> (fn(ArrayView1<f64>, ArrayView1<f64>) -> f64, fn(ArrayView1<f64>, ArrayView1<f64>) -> Array1<f64>) {
    match t {
        Type::MSE => (mse, mse_prime),
        Type::MAE => (mae, mae_prime)
    }
}

pub fn mse(y_true: ArrayView1<f64>, y_pred: ArrayView1<f64>) -> f64 {
    let mut temp = &y_true - &y_pred;
    temp.mul_assign(&temp.clone());
    let mut sum = 0.0;
    for i in 0..temp.len() {
        sum += temp.get(i).unwrap();
    }
    sum / temp.len() as f64
}

pub fn mse_prime(y_true: ArrayView1<f64>, y_pred: ArrayView1<f64>) -> Array1<f64> {
    let temp = &y_true - &y_pred;
    temp / (y_true.len() as f64 / 2.0)
}

pub fn mae(y_true: ArrayView1<f64>, y_pred: ArrayView1<f64>) -> f64 {
    let temp = &y_true - &y_pred;
    let mut sum = 0.0;
    for i in 0..temp.len() {
        sum += temp.get(i).unwrap().abs();
    }
    sum / temp.len() as f64
}

pub fn mae_prime(y_true: ArrayView1<f64>, y_pred: ArrayView1<f64>) -> Array1<f64> {
    let mut result = Array1::zeros(y_true.raw_dim());
    for i in 0..result.len() {
        if y_true.get(i).unwrap() < y_pred.get(i).unwrap() {
            *result.get_mut(i).unwrap() = 1.0;
        } else {
            *result.get_mut(i).unwrap() = -1.0;
        }
    }
    result
}
